/*
 * Copyright (c) 2020-2020,2023 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                    Gianluca Corsini on Mon Nov 23 2020
 */
#ifndef H_LIBDYNAMIXEL
#define H_LIBDYNAMIXEL

#define DX_DEFAULT_ID		255
#define DX_BROADCAST_ID		0xfe
#define DX_DEFAULT_EEPROM	""

#include <sys/types.h>
#include <stdint.h>

enum dx_cmd {
  DX_PING =		0x01,
  DX_READ =		0x02,
  DX_WRITE =		0x03,
  DX_REBOOT =		0x08,
  DX_SYNC_READ =	0x82,
  DX_SYNC_WRITE =	0x83
};

enum dx_model {
  DX_MX28T,
  DX_MX64T,
  DX_MAGNET
};

enum dx_mode {
  DX_TORQUE_MODE = 0,
  DX_VELOCITY_MODE = 1,
  DX_POSITION_MODE = 3, /* default for MX64T */
  DX_EXT_POSITION_MODE = 4,
  DX_TORQUE_POSITION_MODE = 5,
};

struct dynamixelstate_motor_s {
  double effort, velocity, position;
};

struct pi_s {
  double kP, kI;
 };

struct pidff_s {
  double kP, kI, kD, kFF1, kFF2;
};

struct dynamixel_s;
struct dynamixel_pty_s;

#ifdef __cplusplus
extern "C" {
#endif

  /* sim.c */
  struct dynamixel_s *
		dynamixel_init(enum dx_model model, uint8_t id,
                        const char *eeprom_path);
  void		dynamixel_fini(struct dynamixel_s *dynamixel);
  int		dynamixel_get_msg(struct dynamixel_pty_s *pty);
  int		dynamixel_process_msg(const struct dynamixel_s *dynamixel,
                        struct dynamixel_pty_s *pty);

  /* pty.c */
  struct dynamixel_pty_s *
		dynamixel_init_pty(void);
  void		dynamixel_fini_pty(struct dynamixel_pty_s *pty);
  int		dynamixel_open_pty(struct dynamixel_pty_s *pty,
                        const char *path);
  const char *	dynamixel_pty_name(struct dynamixel_pty_s *pty);
  int		dynamixel_wait_msg(struct dynamixel_pty_s *pty, int timeoutms);

  /* mem.c */
  const char *	dynamixel_get_eeprom_path(const struct dynamixel_s *dynamixel);
  uint8_t	dynamixel_get_id(const struct dynamixel_s *dynamixel);
  struct dynamixelstate_motor_s *
		dynamixel_get_motor_ref(struct dynamixel_s * dynamixel);
  void		dynamixel_set_motor_state(struct dynamixel_s *dynamixel,
                        struct dynamixelstate_motor_s *state);
  struct pi_s *	dynamixel_get_velocity_pi(struct dynamixel_s *dynamixel);
  struct pidff_s *
		dynamixel_get_position_pidff(struct dynamixel_s *dynamixel);
  enum dx_mode	dynamixel_get_op_mode(struct dynamixel_s *dynamixel);

#ifdef __cplusplus
}
#endif


#endif /* H_LIBDYNAMIXEL */
